#include "xmedia_config.h"
#include <iostream>

xmedia_config::xmedia_config()
{
    this->Settings = new QSettings("/home/xMedia/config.ini", QSettings::IniFormat, 0);
    this->reload();
}

xmedia_config::~xmedia_config()
{
    delete this->Settings;
}

void xmedia_config::save()
{
    this->Settings->beginGroup("Network");

    this->Settings->beginWriteArray("Adapters");
    for(int i = 0; i < this->networks.size(); i++)
    {
        this->Settings->setArrayIndex(i);
        this->Settings->setValue("DHCP", this->networks[i].dhcp);
        this->Settings->setValue("Address", this->networks[i].address);
        this->Settings->setValue("Subnet", this->networks[i].subnet);
        this->Settings->setValue("Gateway", this->networks[i].gateway);
        this->Settings->setValue("DNS", this->networks[i].dns);
    }
    this->Settings->endArray();

    this->Settings->endGroup();

    this->Settings->beginGroup("Media");

    this->Settings->beginWriteArray("Shares");
    for(int i = 0; i < this->shares.size(); i++)
    {
        this->Settings->setArrayIndex(i);
        this->Settings->setValue("Index", this->shares[i].index);
        this->Settings->setValue("Address", this->shares[i].address);
        this->Settings->setValue("Name", this->shares[i].name);
    }
    this->Settings->endArray();

    this->Settings->beginWriteArray("Stations");
    for(int i = 0; i < this->stations.size(); i++)
    {
        this->Settings->setArrayIndex(i);
        this->Settings->setValue("Index", this->stations[i].index);
        this->Settings->setValue("Address", this->stations[i].address);
        this->Settings->setValue("Name", this->stations[i].name);
    }
    this->Settings->endArray();

    this->Settings->endGroup();

    this->Settings->beginGroup("General");

     QMapIterator<QString, QString> i(this->Parameters);
     while (i.hasNext())
     {
        i.next();
        this->Settings->setValue(i.key(), i.value());
     }

    this->Settings->endGroup();
    this->Settings->sync();
}

void xmedia_config::reload()
{
    this->Settings->beginGroup("Network");

    int size = this->Settings->beginReadArray("Adapters");

    this->networks.clear();

    for(int i = 0; i < size; i++)
    {
        this->Settings->setArrayIndex(i);

        xMediaConfigNetworks network;

        network.dhcp = this->Settings->value("DHCP").toBool();
        network.address = this->Settings->value("Address").toString();
        network.subnet = this->Settings->value("Subnet").toString();
        network.gateway = this->Settings->value("Gateway").toString();
        network.dns = this->Settings->value("DNS").toString();

        this->networks.append(network);
    }
    this->Settings->endArray();

    this->Settings->endGroup();

    this->Settings->beginGroup("Media");

    size = this->Settings->beginReadArray("Shares");

    this->shares.clear();

    for(int i = 0; i < size; i++)
    {
        this->Settings->setArrayIndex(i);

        xMediaConfigShares share;

        share.index = this->Settings->value("Index").toInt();
        share.address = this->Settings->value("Address").toString();
        share.name = this->Settings->value("Name").toString();

        this->shares.append(share);
    }
    this->Settings->endArray();

    this->stations.clear();

    size = this->Settings->beginReadArray("Stations");
    for(int i = 0; i < size; i++)
    {
        this->Settings->setArrayIndex(i);

        xMediaConfigStations station;

        station.index = this->Settings->value("Index").toInt();
        station.address = this->Settings->value("Address").toString();
        station.name = this->Settings->value("Name").toString();

        this->stations.append(station);
    }
    this->Settings->endArray();

    this->Settings->endGroup();

    this->Settings->beginGroup("General");

    QStringList keys = this->Settings->allKeys();

    for(int i = 0; i < keys.size(); i++)
    {
        this->Parameters[keys[i]] = this->Settings->value(keys[i]).toString();
    }

    this->Settings->endGroup();
}

void xmedia_config::setParameter(QString name, QString value)
{
    this->Parameters[name] = value;
}

QString xmedia_config::getParameter(QString name)
{
    if(this->Parameters.contains(name))
    {
        return this->Parameters[name];
    }
    else
    {
        return "";
    }
}
