#include "xmedia_networkconfig.h"
#include <iostream>

xmedia_networkConfig::xmedia_networkConfig()
{
}

QList<xmedia_networkConfigWifiStatus> xmedia_networkConfig::wifiAvailableNetworks()
{
    QList<xmedia_networkConfigWifiStatus> data;

    QString results = getProcessResult("iwlist scanning");

    QStringList interfaces = results.split("\r\n\r\n");

    for(int i = 0; i < interfaces.size(); i++)
    {
        QString interface = interfaces[i];
        QStringList networks = results.split("Cell");

        for(int k = 0; k < networks.size(); k++)
        {
            QString dataset = networks[k];

            if(dataset.contains("Address"))
            {
                xmedia_networkConfigWifiStatus status;

                status.accessPoint      = this->extractString(dataset, "Address:", "\n");
                status.essid            = this->extractString(dataset, "ESSID:\"", "\"");
                status.frequency        = this->extractString(dataset, "Frequency=", "\n");
                status.linkQuality      = this->extractString(dataset, "Quality=", "/").toInt();
                status.linkSignalLevel  = this->extractString(dataset, "Signal level=", "dBm") + "dBm";
                status.linkNoiseLevel   = this->extractString(dataset, "Noise level=", "dBm") + "dBm";
                status.bitRate          = "";
                status.txPower          = "";
                status.secKey           = "";
                status.secMode          = "";

                data.append(status);
            }
        }
    }

    return data;
}
void xmedia_networkConfig::wifiConfigureNetwork(QString interface, QString essid, QString key, QString datarate)
{
    getProcessResult("iwconfig " + interface + " essid " + essid);
    getProcessResult("iwconfig " + interface + " key " + key);
    getProcessResult("iwconfig " + interface + " rate " + datarate);
}

xmedia_networkConfigWifiStatus xmedia_networkConfig::wifiNetworkStatus()
{
    xmedia_networkConfigWifiStatus status;

    QString results = getProcessResult("iwconfig");

    QStringList interfaces = results.split("\r\n\r\n");

    for(int i = 0; i < interfaces.size(); i++)
    {
        QString dataset = interfaces[i];

        if(dataset.contains("IEEE"))
        {
            status.accessPoint      = this->extractString(dataset, "Access Point:", "\n");
            status.essid            = this->extractString(dataset, "ESSID:\"", "\"");
            status.frequency        = this->extractString(dataset, "Frequency:", "GHz") + "GHz";
            status.linkQuality      = this->extractString(dataset, "Quality=", "/").toInt();
            status.linkSignalLevel  = this->extractString(dataset, "Signal level=", " ") + "dBm";
            status.linkNoiseLevel   = this->extractString(dataset, "Noise level=", " ") + "dBm";
            status.bitRate          = (dataset.contains("Bit Rate=")?this->extractString(dataset, "Bit Rate=", "/s"):this->extractString(dataset, "Bit Rate:", "/s"));
            status.bitRate          += "/s";
            status.txPower          = this->extractString(dataset, "Tx-Power=", "dBm") + "dBm";
            status.secKey           = this->extractString(dataset, "Encryption key:", " ").replace("-", "");
            status.secMode          = this->extractString(dataset, "Security mode:", " ").replace("-", "");
        }
    }

    return status;
}


QList<xmedia_networkConfigNetworkStatus> xmedia_networkConfig::networkStatus()
{
    QList<xmedia_networkConfigNetworkStatus> data;

    QString route_results = getProcessResult("route -e -n");

    QString results = getProcessResult("ifconfig");

    QStringList interfaces = results.split("\n\n");

    for(int i = 0; i < interfaces.size(); i++)
    {
        QString dataset = interfaces[i];

        if(dataset.contains("HWaddr"))
        {
            xmedia_networkConfigNetworkStatus status;

            status.name             = "eth" + this->extractString(dataset, "eth", " ");
            status.mac              = this->extractString(dataset, "HWaddr ", "\n");
            status.ip               = this->extractString(dataset, "inet addr:", " ");
            status.subnet           = this->extractString(dataset, "Mask:", "\n");
            status.bytesRX          = this->extractString(dataset, "RX bytes:", " ").toLong();
            status.bytesTX          = this->extractString(dataset, "TX bytes:", " ").toLong();

            QStringList routes = route_results.split("\n");

            for(int i = 0; i < routes.size(); i++)
            {
                QString route = routes[i];

                if(route.contains(status.name) && route.contains("UG")) //Gateways
                {
                    status.gateway          = this->extractString(route, "0.0.0.0\t\t", "0.0.0.0").remove(" ");
                }
            }

            data.append(status);
        }
    }

    return data;
}

void xmedia_networkConfig::networkConfigure(QString interface, bool useDhcp, QString ip, QString subnet, QString gateway, QString dns)
{
    if(!useDhcp)
    {
        getProcessResult("ifconfig " + interface + " " + ip +" netmask " + subnet + " &");
    }
    else
    {
        getProcessResult("udhcpc -i " + interface + " &");
    }

    //TODO Route
}

QString xmedia_networkConfig::getProcessResult(QString process)
{
    QString data = "";
    QProcess *proc = new QProcess();
    proc->start(process);
    proc->waitForFinished(30000);
    while(proc->canReadLine())
    {
        data += proc->readLine();
    }
    delete proc;

    return data;
}

 QString xmedia_networkConfig::extractString(QString data, QString before, QString after)
 {

     int p1 = data.indexOf(before)+before.length();
     if(after != NULL)
     {
         int p2 = data.indexOf(after, p1)+after.length();
         return data.mid(p1, p2-p1-after.length());
     }
     else
     {
         return data.mid(p1);
     }
 }
