#include "xserver.h"
#include <iostream>

using namespace std;

xServer::xServer(int argc, char *argv[])
{
    cout<<"Content-Type: text/html"<<endl;
    cout<<"Pragma: no-cache"<<endl;
    cout<<endl;

    this->parseParameters();
    this->setTempalateVariable("xwebpath", "../xweb/");

    this->loadTemplateFileAndPrint("../xweb/templates/head.tpl.htm");

    if(this->Parameters.contains("page"))
    {
        if(this->Parameters["page"] == "main")
        {
             this->pageMain();
        }
        else if(this->Parameters["page"] == "wifi")
        {
             this->pageWifi();
        }
        else if(this->Parameters["page"] == "network")
        {
             this->pageNetwork();
        }
    }
    else
    {
        this->pageMain();
    }

    this->loadTemplateFileAndPrint("../xweb/templates/foot.tpl.htm");
}

void xServer::parseParameters()
{
    QString query = getenv("QUERY_STRING");

    this->Parameters.clear();

    while(query.length() > 0)
    {
        int name_start = 0;
        int name_end = query.indexOf("=");
        int value_start = name_end + 1;
        int value_end = query.indexOf("&");
        value_end = value_end!=-1?value_end:query.length();

        QString name = query.mid(name_start, name_end - name_start);
        QString value = query.mid(value_start, value_end - value_start);

        this->Parameters[name] = value;
        query = query.mid(value_end+1);
    }
}

void xServer::setTempalateVariable(QString key, QString value)
{
    this->TemplateVariables[key] = value;
}

void xServer::loadTemplateFileAndPrint(QString lfile)
{
    cout<<qPrintable(loadTemplateFile(lfile))<<endl;
}

QString xServer::loadTemplateFile(QString lfile)
{
    QString data = "";

     QFile file(lfile);
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
         return "";

     QTextStream in(&file);
     while (!in.atEnd()) {
         QString line = in.readLine();
         data += replaceTemplatePlaceholders(line);
     }

     return data;
}

QString xServer::replaceTemplatePlaceholders(QString data)
{
     QMapIterator<QString, QString> i(this->TemplateVariables);
     while (i.hasNext()) {
         i.next();

         data.replace("<?" + i.key() + "?>", i.value());
     }
     return data;
}

void xServer::pageMain()
{
    QString uptime = this->getProcessResult("uptime");
    QString cpuinfo = this->getProcessResult("cat /proc/cpuinfo");
    QString meminfo = this->getProcessResult("cat /proc/meminfo");
    QString version = this->getProcessResult("cat /proc/version");

    this->setTempalateVariable("uptime", this->extractString(uptime, "up", ","));
    this->setTempalateVariable("load", this->extractString(uptime, "age:", NULL));
    this->setTempalateVariable("core_chip", this->extractString(cpuinfo, "chip type\t:", "\n"));
    this->setTempalateVariable("core_arch", this->extractString(cpuinfo, "cpu arch\t:", "\n"));
    this->setTempalateVariable("core_speed", this->extractString(cpuinfo, "cpu MHz\t\t:", "\n"));
    this->setTempalateVariable("core_mips", this->extractString(cpuinfo, "bogomips\t:", "\n"));
    this->setTempalateVariable("core_memory", this->extractString(meminfo, "MemTotal:", "\n"));
    this->setTempalateVariable("core_memory_free", this->extractString(meminfo, "MemFree:", "\n"));
    this->setTempalateVariable("kernel_version", this->extractString(version, "Linux version", "("));
    this->setTempalateVariable("kernel_build", this->extractString(version, "#", " "));
    this->setTempalateVariable("kernel_user", this->extractString(version, this->extractString(version, "Linux version", "(") + "(", ")"));
    this->setTempalateVariable("kernel_compiler", this->extractString(version, "(gcc version", ")"));
    this->setTempalateVariable("kernel_date", this->extractString(version, "#" + this->extractString(version, "#", " "), NULL));

    this->loadTemplateFileAndPrint("../xweb/templates/main.tpl.htm");
}

void xServer::pageWifi()
{
    xmedia_networkConfig *nconfig = new xmedia_networkConfig();
    xmedia_config *config = new xmedia_config();

    if(this->Parameters.contains("save"))
    {
        config->setParameter("Wifi/Essid", this->Parameters["essid"]);
        config->setParameter("Wifi/Key", this->Parameters["key"]);
        config->save();
        nconfig->wifiConfigureNetwork("eth1", this->Parameters["essid"], this->Parameters["key"], "54M");
    }


    QString data = "";

    xmedia_networkConfigWifiStatus status = nconfig->wifiNetworkStatus();

    this->setTempalateVariable("essid", status.essid);
    this->setTempalateVariable("frequency", status.frequency);
    this->setTempalateVariable("accessPoint", status.accessPoint);
    this->setTempalateVariable("linkQuality", QString::number(status.linkQuality) + "%");
    this->setTempalateVariable("linkSignalLevel", status.linkSignalLevel);
    this->setTempalateVariable("linkNoiseLevel", status.linkNoiseLevel);
    this->setTempalateVariable("bitRate", status.bitRate);
    this->setTempalateVariable("txPower", status.txPower);
    this->setTempalateVariable("secKey", status.secKey);
    this->setTempalateVariable("secMode", status.secMode);

    data = this->loadTemplateFile("../xweb/templates/wifi_status.tpl.htm");

    this->setTempalateVariable("wifiStatus", data);

    QList<xmedia_networkConfigWifiStatus> networks = nconfig->wifiAvailableNetworks();

    data = "";

    for(int i = 0; i < networks.size(); i++)
    {
        this->setTempalateVariable("id", QString::number(i+1));
        this->setTempalateVariable("essid", networks[i].essid);
        this->setTempalateVariable("frequency", networks[i].frequency);
        this->setTempalateVariable("accessPoint", networks[i].accessPoint);
        this->setTempalateVariable("linkQuality", QString::number(networks[i].linkQuality) + "%");
        this->setTempalateVariable("linkSignalLevel", networks[i].linkSignalLevel);
        this->setTempalateVariable("linkNoiseLevel", networks[i].linkNoiseLevel);

        data += this->loadTemplateFile("../xweb/templates/wifi_scan.tpl.htm");
    }

    this->setTempalateVariable("wifiScan", data);

    QString essid = config->getParameter("Wifi/Essid");
    QString key = config->getParameter("Wifi/Key");

    this->setTempalateVariable("wifiEssid", essid);
    this->setTempalateVariable("wifiKey", key);

    this->loadTemplateFileAndPrint("../xweb/templates/wifi.tpl.htm");
}

void xServer::pageNetwork()
{
    xmedia_networkConfig *nconfig = new xmedia_networkConfig();
    xmedia_config *config = new xmedia_config();

    QString data = "";
    QString configuration = "";

    QList<xmedia_networkConfigNetworkStatus> networks = nconfig->networkStatus();

    for(int i = 0; i < networks.size(); i++)
    {
        this->setTempalateVariable("name", networks[i].name);
        this->setTempalateVariable("mac", networks[i].mac);
        this->setTempalateVariable("ip", networks[i].ip);
        this->setTempalateVariable("subnet", networks[i].subnet);
        this->setTempalateVariable("gateway", networks[i].gateway);
        this->setTempalateVariable("bytesRX", QString::number(networks[i].bytesRX/1024/1024) + " MiB");
        this->setTempalateVariable("bytesTX", QString::number(networks[i].bytesTX/1024/1024) + " MiB");

        data += this->loadTemplateFile("../xweb/templates/network_status.tpl.htm");

    }

    for(int i = 0; i < config->networks.size(); i++)
    {
        xMediaConfigNetworks network = config->networks[i];

        this->setTempalateVariable("networkAddress", network.address);
        this->setTempalateVariable("networkSubnet", network.subnet);
        this->setTempalateVariable("networkGateway", network.gateway);
        this->setTempalateVariable("networkDhcp", network.dhcp?"checked":"");
        this->setTempalateVariable("networkDns", network.dns);
        QString tmp = "";
        tmp.setNum(i+1);
        this->setTempalateVariable("networkInterface", "eth" + tmp);

        configuration += this->loadTemplateFile("../xweb/templates/network_config.tpl.htm");
    }

    this->setTempalateVariable("networkConfig", configuration);
    this->setTempalateVariable("networkStatus", data);

    this->loadTemplateFileAndPrint("../xweb/templates/network.tpl.htm");
}


QString xServer::getProcessResult(QString process)
{
    QString data = "";
    QProcess *proc = new QProcess();
    proc->start(process);
    proc->waitForFinished(30000);
    while(proc->canReadLine())
    {
        data += proc->readLine();
    }
    delete proc;

    return data;
}

 QString xServer::extractString(QString data, QString before, QString after)
 {

     int p1 = data.indexOf(before)+before.length();
     if(after != NULL)
     {
         int p2 = data.indexOf(after, p1)+after.length();
         return data.mid(p1, p2-p1-after.length());
     }
     else
     {
         return data.mid(p1);
     }
 }
